

PLOTLY_POLAR = r'https://raw.githubusercontent.com/plotly/datasets/master/polar_dataset.csv'
PLOTLY_POINTS = r'https://raw.githubusercontent.com/plotly/datasets/master/3d-line1.csv'

data_line_3d = r'https://raw.githubusercontent.com/plotly/datasets/master/_3d-line-plot.csv'
data_mesh_3d = r'https://raw.githubusercontent.com/plotly/datasets/master/mesh_dataset.txt'

PLOTLY_APPLE_PRICES = r'https://raw.githubusercontent.com/plotly/datasets/master/finance-charts-apple.csv'

GEO_US_CITIES = 'https://raw.githubusercontent.com/plotly/datasets/master/2014_us_cities.csv'
GEO_US_RAINS = 'https://raw.githubusercontent.com/plotly/datasets/master/2015_06_30_precipitation.csv'
GEO_EARTHQUAKES = 'https://raw.githubusercontent.com/plotly/datasets/master/earthquakes-23k.csv'

GEO_COVID19_COUNTRIES = "'https://covid.ourworldindata.org/data/total_cases.csv'"

AIRPORT_TRAFFIC = 'https://raw.githubusercontent.com/plotly/datasets/master/2011_february_us_airport_traffic.csv'
PLOTLY_WEBGL_POLAR = 'https://raw.githubusercontent.com/plotly/datasets/master/hobbs-pearson-trials.csv'

BANKING_DATA = 'https://raw.githubusercontent.com/juliencohensolal/BankMarketing/master/rawData/bank-additional-full.csv'


#####################################################################################################################
# AG GRID
# website: https://github.com/ag-grid
#
OLYMPIC_WINNERS_SMALL = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/olympicWinnersSmall.json'
OLYMPIC_WINNERS_FULL = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/olympicWinners.json'
MONTLY_SALES = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/sample-data/monthlySales.json'
STOCKS = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/stocks.json'
BLOG_OBJECT = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/blogs.json'
WEATHER_ENGLAND = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/weather_se_england.json'
AGGRID_OBJECTS_NESTED = 'https://raw.githubusercontent.com/ag-grid/ag-grid/master/grid-packages/ag-grid-docs/src/feature-overlays.json'

SP500W_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/sp500w.csv'
GE_SHARE_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/GE.csv'
MSFT_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/MSFT.csv'
BODY_DATA = 'https://raw.githubusercontent.com/ag-grid/ag-grid-charts-example/master/data/body-data.csv'


# https://bl.ocks.org/d3noob
DC_QUAKES = 'http://bl.ocks.org/d3noob/raw/6077996/45425ba06fc3c30ed3c7492d1042352d3dad0f18/quakes.csv'

#https://earthquake.usgs.gov/earthquakes/feed/v1.0/csv.php
DATA_EARTHQUAKE = 'https://earthquake.usgs.gov/earthquakes/feed/v1.0/summary/all_week.csv'
WORLD_INDEX = 'https://visjs.github.io/vis-graph3d/examples/graph3d/rotateAxisLabels/world_index_data_small.json'

AIRLINE_SAFETY = r"https://raw.githubusercontent.com/fivethirtyeight/data/master/airline-safety/airline-safety.csv"
MURDEERS_2015 = r"https://raw.githubusercontent.com/fivethirtyeight/data/master/murder_2016/murder_2015_final.csv"
US_BIRTHS = r"https://raw.githubusercontent.com/fivethirtyeight/data/master/births/US_births_2000-2014_SSA.csv"
ZIKA_VIRUS = "https://raw.githubusercontent.com/BuzzFeedNews/zika-data/master/data/raw/paho-who/paho-who-cases-reported-latest.csv"
FIREARM_STATS_US = "https://raw.githubusercontent.com/BuzzFeedNews/nics-firearm-background-checks/master/data/nics-firearm-background-checks.csv"

CARS = 'https://perso.telecom-paristech.fr/eagan/class/igr204/data/cars.csv'
CAUSE_OF_DEATH = 'https://perso.telecom-paristech.fr/eagan/class/igr204/data/CausesOfDeath_France_2001-2008.csv'
BABY_NAMES = 'https://perso.telecom-paristech.fr/eagan/class/igr204/data/nat1900-2017.tsv'
HAPPINESS_STATS = "https://perso.telecom-paristech.fr/eagan/class/igr204/data/happiness.csv"

TABULATOR_DATA = 'http://tabulator.info/sample_data/file_load/data.json'

DATATABLE_ARRAY_URL = 'https://datatables.net/examples/ajax/data/arrays.txt'
DATATABLE_ARRAY_NESTED_URL = 'https://datatables.net/examples/ajax/data/objects_subarrays.txt'
DATATABLE_OBJECTS_URL = 'https://datatables.net/examples/ajax/data/objects.txt'
DATATABLE_OBJECTS_NESTED_URL = 'https://datatables.net/examples/ajax/data/objects_deep.txt'
DATATABLE_ORTHOGONAL_URL = 'https://datatables.net/examples/ajax/data/orthogonal.txt'
DATATABLE_OBJECT_ARRAY_URL = 'https://datatables.net/examples/ajax/data/objects_root_array.txt'

COVID_ECDC_URL = 'https://www.ecdc.europa.eu/sites/default/files/documents/COVID-19-geographic-disbtribution-worldwide-2020-03-21.xlsx'
COVID_TIME_SERIES = 'https://github.com/CSSEGISandData/COVID-19/tree/master/csse_covid_19_data/csse_covid_19_time_series'
COVID_DAILY_DATA = 'https://github.com/CSSEGISandData/COVID-19/tree/master/csse_covid_19_data/csse_covid_19_daily_reports'