"""
Related Pages:

		https://www.w3schools.com/charsets/ref_emoji_smileys.asp
"""

GRINNING_FACE = '&#128512'
GRINNING_FACE_WITH_SMILING_EYES = '&#128513'
FACE_WITH_TEARS_OF_JOY = '&#128514'
SMILING_FACE_WITH_OPEN_MOUTH = '&#128515'
SMILING_FACE_WITH_OPEN_MOUTH_AND_SMILING_EYES = '&#128516'
SMILING_FACE_WITH_OPEN_MOUTH_AND_COLD_SWEAT = '&#128517'
SMILING_FACE_WITH_OPEN_MOUTH_AND_TIGHTLY_CLOSED_EYES = '&#128518'
SMILING_FACE_WITH_HALO = '&#128519'
SMILING_FACE_WITH_HORNS = '&#128520'
WINKING_FACE = '&#128521'
SMILING_FACE_WITH_SMILING_EYES = '&#128522'
FACE_SAVOURING_DELICIOUS_FOOD = '&#128523'
RELIEVED_FACE = '&#128524'
SMILING_FACE_WITH_HEART_SHAPED_EYES = '&#128525'
SMILING_FACE_WITH_SUNGLASSES = '&#128526'
SMIRKING_FACE = '&#128527'
NEUTRAL_FACE = '&#128528'
EXPRESSIONLESS_FACE = '&#128529'
UNAMUSED_FACE = '&#128530'
FACE_WITH_COLD_SWEAT = '&#128531'
PENSIVE_FACE = '&#128532'
CONFUSED_FACE = '&#128533'
CONFOUNDED_FACE = '&#128534'
KISSING_FACE = '&#128535'
FACE_THROWING_A_KISS = '&#128536'
KISSING_FACE_WITH_SMILING_EYES = '&#128537'
KISSING_FACE_WITH_CLOSED_EYES = '&#128538'
FACE_WITH_STUCK_OUT_TONGUE = '&#128539'
FACE_WITH_STUCK_OUT_TONGUE_AND_WINKING_EYE = '&#128540'
FACE_WITH_STUCK_OUT_TONGUE_AND_TIGHTLY_CLOSED_EYES = '&#128541'
DISAPPOINTED_FACE = '&#128542'
WORRIED_FACE = '&#128543'
ANGRY_FACE = '&#128544'
POUTING_FACE = '&#128545'
CRYING_FACE = '&#128546'
PERSEVERING_FACE = '&#128547'
FACE_WITH_LOOK_OF_TRIUMPH = '&#128548'
DISAPPOINTED_BUT_RELIEVED_FACE = '&#128549'
FROWNING_FACE_WITH_OPEN_MOUTH = '&#128550'
ANGUISHED_FACE = '&#128551'
FEARFUL_FACE = '&#128552'
WEARY_FACE = '&#128553'
SLEEPY_FACE = '&#128554'
TIRED_FACE = '&#128555'
GRIMACING_FACE = '&#128556'
LOUDLY_CRYING_FACE = '&#128557'
FACE_WITH_OPEN_MOUTH = '&#128558'
HUSHED_FACE = '&#128559'
FACE_WITH_OPEN_MOUTH_AND_COLD_SWEAT = '&#128560'
FACE_SCREAMING_IN_FEAR = '&#128561'
ASTONISHED_FACE = '&#128562'
FLUSHED_FACE = '&#128563'
SLEEPING_FACE = '&#128564'
DIZZY_FACE = '&#128565'
FACE_WITHOUT_MOUTH = '&#128566'
FACE_WITH_MEDICAL_MASK = '&#128567'
