"""
Related Pages:

		https://www.w3schools.com/charsets/ref_utf_letterlike.asp
"""

ACCOUNT_OF = '&#8448'
ADDRESSED_TO_THE_SUBJECT = '&#8449'
DOUBLE_STRUCK_CAPITAL_C = '&#8450'
DEGREE_CELSIUS = '&#8451'
CENTRE_LINE_SYMBOL = '&#8452'
CARE_OF = '&#8453'
CADA_UNA = '&#8454'
EULER_CONSTANT = '&#8455'
SCRUPLE = '&#8456'
DEGREE_FAHRENHEIT = '&#8457'
SCRIPT_SMALL_G = '&#8458'
SCRIPT_CAPITAL_H = '&#8459'
BLACK_LETTER_CAPITAL_H = '&#8460'
DOUBLE_STRUCK_CAPITAL_H = '&#8461'
PLANCK_CONSTANT = '&#8462'
PLANCK_CONSTANT_OVER_TWO_PI = '&#8463'
SCRIPT_CAPITAL_I = '&#8464'
BLACK_LETTER_CAPITAL_I = '&#8465'
SCRIPT_CAPITAL_L = '&#8466'
SCRIPT_SMALL_L = '&#8467'
L_B_BAR_SYMBOL = '&#8468'
DOUBLE_STRUCK_CAPITAL_N = '&#8469'
NUMERO_SIGN = '&#8470'
SOUND_RECORDING_COPYRIGHT = '&#8471'
SCRIPT_CAPITAL_P = '&#8472'
DOUBLE_STRUCK_CAPITAL_P = '&#8473'
DOUBLE_STRUCK_CAPITAL_Q = '&#8474'
SCRIPT_CAPITAL_R = '&#8475'
BLACK_LETTER_CAPITAL_R = '&#8476'
DOUBLE_STRUCK_CAPITAL_R = '&#8477'
PRESCRIPTION_TAKE = '&#8478'
RESPONSE = '&#8479'
SERVICE_MARK = '&#8480'
TELEPHONE_SIGN = '&#8481'
TRADE_MARK_SIGN = '&#8482'
VERSICLE = '&#8483'
DOUBLE_STRUCK_CAPITAL_Z = '&#8484'
OUNCE_SIGN = '&#8485'
OHM_SIGN = '&#8486'
INVERTED_OHM_SIGN = '&#8487'
BLACK_LETTER_CAPITAL_Z = '&#8488'
TURNED_GREEK_SMALL_LETTER_IOTA = '&#8489'
KELVIN_SIGN = '&#8490'
ANGSTROM_SIGN = '&#8491'
SCRIPT_CAPITAL_B = '&#8492'
BLACK_LETTER_CAPITAL_C = '&#8493'
ESTIMATED_SYMBOL = '&#8494'
SCRIPT_SMALL_E = '&#8495'
SCRIPT_CAPITAL_E = '&#8496'
SCRIPT_CAPITAL_F = '&#8497'
TURNED_CAPITAL_F = '&#8498'
SCRIPT_CAPITAL_M = '&#8499'
SCRIPT_SMALL_O = '&#8500'
ALEF_SYMBOL = '&#8501'
BET_SYMBOL = '&#8502'
GIMEL_SYMBOL = '&#8503'
DALET_SYMBOL = '&#8504'
INFORMATION_SOURCE = '&#8505'
ROTATED_CAPITAL_Q = '&#8506'
FACSIMILE_SIGN = '&#8507'
DOUBLE_STRUCK_SMALL_PI = '&#8508'
DOUBLE_STRUCK_SMALL_GAMMA = '&#8509'
DOUBLE_STRUCK_CAPITAL_GAMMA = '&#8510'
DOUBLE_STRUCK_CAPITAL_PI = '&#8511'
DOUBLE_STRUCK_N_ARY_SUMMATION = '&#8512'
TURNED_SANS_SERIF_CAPITAL_G = '&#8513'
TURNED_SANS_SERIF_CAPITAL_L = '&#8514'
REVERSED_SANS_SERIF_CAPITAL_L = '&#8515'
TURNED_SANS_SERIF_CAPITAL_Y = '&#8516'
DOUBLE_STRUCK_ITALIC_CAPITAL_D = '&#8517'
DOUBLE_STRUCK_ITALIC_SMALL_D = '&#8518'
DOUBLE_STRUCK_ITALIC_SMALL_E = '&#8519'
DOUBLE_STRUCK_ITALIC_SMALL_I = '&#8520'
DOUBLE_STRUCK_ITALIC_SMALL_J = '&#8521'
PROPERTY_LINE = '&#8522'
TURNED_AMPERSAND = '&#8523'
PER_SIGN = '&#8524'
AKTIESELSKAB = '&#8525'
TURNED_SMALL_F = '&#8526'
SYMBOL_FOR_SAMARITAN_SOURCE = '&#8527'
