"""
Related Pages:

		https://www.w3schools.com/charsets/ref_utf_arrows.asp
"""

LEFTWARDS_ARROW = '&#8592'
UPWARDS_ARROW = '&#8593'
RIGHTWARDS_ARROW = '&#8594'
DOWNWARDS_ARROW = '&#8595'
LEFT_RIGHT_ARROW = '&#8596'
UP_DOWN_ARROW = '&#8597'
NORTH_WEST_ARROW = '&#8598'
NORTH_EAST_ARROW = '&#8599'
SOUTH_EAST_ARROW = '&#8600'
SOUTH_WEST_ARROW = '&#8601'
LEFTWARDS_ARROW_WITH_STROKE = '&#8602'
RIGHTWARDS_ARROW_WITH_STROKE = '&#8603'
LEFTWARDS_WAVE_ARROW = '&#8604'
RIGHTWARDS_WAVE_ARROW = '&#8605'
LEFTWARDS_TWO_HEADED_ARROW = '&#8606'
UPWARDS_TWO_HEADED_ARROW = '&#8607'
RIGHTWARDS_TWO_HEADED_ARROW = '&#8608'
DOWNWARDS_TWO_HEADED_ARROW = '&#8609'
LEFTWARDS_ARROW_WITH_TAIL = '&#8610'
RIGHTWARDS_ARROW_WITH_TAIL = '&#8611'
LEFTWARDS_ARROW_FROM_BAR = '&#8612'
UPWARDS_ARROW_FROM_BAR = '&#8613'
RIGHTWARDS_ARROW_FROM_BAR = '&#8614'
DOWNWARDS_ARROW_FROM_BAR = '&#8615'
UP_DOWN_ARROW_WITH_BASE = '&#8616'
LEFTWARDS_ARROW_WITH_HOOK = '&#8617'
RIGHTWARDS_ARROW_WITH_HOOK = '&#8618'
LEFTWARDS_ARROW_WITH_LOOP = '&#8619'
RIGHTWARDS_ARROW_WITH_LOOP = '&#8620'
LEFT_RIGHT_WAVE_ARROW = '&#8621'
LEFT_RIGHT_ARROW_WITH_STROKE = '&#8622'
DOWNWARDS_ZIGZAG_ARROW = '&#8623'
UPWARDS_ARROW_WITH_TIP_LEFTWARDS = '&#8624'
UPWARDS_ARROW_WITH_TIP_RIGHTWARDS = '&#8625'
DOWNWARDS_ARROW_WITH_TIP_LEFTWARDS = '&#8626'
DOWNWARDS_ARROW_WITH_TIP_RIGHTWARDS = '&#8627'
RIGHTWARDS_ARROW_WITH_CORNER_DOWNWARDS = '&#8628'
DOWNWARDS_ARROW_WITH_CORNER_LEFTWARDS = '&#8629'
ANTICLOCKWISE_TOP_SEMICIRCLE_ARROW = '&#8630'
CLOCKWISE_TOP_SEMICIRCLE_ARROW = '&#8631'
NORTH_WEST_ARROW_TO_LONG_BAR = '&#8632'
LEFTWARDS_ARROW_TO_BAR_OVER_RIGHTWARDS_ARROW_TO_BAR = '&#8633'
ANTICLOCKWISE_OPEN_CIRCLE_ARROW = '&#8634'
CLOCKWISE_OPEN_CIRCLE_ARROW = '&#8635'
LEFTWARDS_HARPOON_WITH_BARB_UPWARDS = '&#8636'
LEFTWARDS_HARPOON_WITH_BARB_DOWNWARDS = '&#8637'
UPWARDS_HARPOON_WITH_BARB_RIGHTWARDS = '&#8638'
UPWARDS_HARPOON_WITH_BARB_LEFTWARDS = '&#8639'
RIGHTWARDS_HARPOON_WITH_BARB_UPWARDS = '&#8640'
RIGHTWARDS_HARPOON_WITH_BARB_DOWNWARDS = '&#8641'
DOWNWARDS_HARPOON_WITH_BARB_RIGHTWARDS = '&#8642'
DOWNWARDS_HARPOON_WITH_BARB_LEFTWARDS = '&#8643'
RIGHTWARDS_ARROW_OVER_LEFTWARDS_ARROW = '&#8644'
UPWARDS_ARROW_LEFTWARDS_OF_DOWNWARDS_ARROW = '&#8645'
LEFTWARDS_ARROW_OVER_RIGHTWARDS_ARROW = '&#8646'
LEFTWARDS_PAIRED_ARROWS = '&#8647'
UPWARDS_PAIRED_ARROWS = '&#8648'
RIGHTWARDS_PAIRED_ARROWS = '&#8649'
DOWNWARDS_PAIRED_ARROWS = '&#8650'
LEFTWARDS_HARPOON_OVER_RIGHTWARDS_HARPOON = '&#8651'
RIGHTWARDS_HARPOON_OVER_LEFTWARDS_HARPOON = '&#8652'
LEFTWARDS_DOUBLE_ARROW_WITH_STROKE = '&#8653'
LEFT_RIGHT_DOUBLE_ARROW_WITH_STROKE = '&#8654'
RIGHTWARDS_DOUBLE_ARROW_WITH_STROKE = '&#8655'
LEFTWARDS_DOUBLE_ARROW = '&#8656'
UPWARDS_DOUBLE_ARROW = '&#8657'
RIGHTWARDS_DOUBLE_ARROW = '&#8658'
DOWNWARDS_DOUBLE_ARROW = '&#8659'
LEFT_RIGHT_DOUBLE_ARROW = '&#8660'
UP_DOWN_DOUBLE_ARROW = '&#8661'
NORTH_WEST_DOUBLE_ARROW = '&#8662'
NORTH_EAST_DOUBLE_ARROW = '&#8663'
SOUTH_EAST_DOUBLE_ARROW = '&#8664'
SOUTH_WEST_DOUBLE_ARROW = '&#8665'
LEFTWARDS_TRIPLE_ARROW = '&#8666'
RIGHTWARDS_TRIPLE_ARROW = '&#8667'
LEFTWARDS_SQUIGGLE_ARROW = '&#8668'
RIGHTWARDS_SQUIGGLE_ARROW = '&#8669'
UPWARDS_ARROW_WITH_DOUBLE_STROKE = '&#8670'
DOWNWARDS_ARROW_WITH_DOUBLE_STROKE = '&#8671'
LEFTWARDS_DASHED_ARROW = '&#8672'
UPWARDS_DASHED_ARROW = '&#8673'
RIGHTWARDS_DASHED_ARROW = '&#8674'
DOWNWARDS_DASHED_ARROW = '&#8675'
LEFTWARDS_ARROW_TO_BAR = '&#8676'
RIGHTWARDS_ARROW_TO_BAR = '&#8677'
LEFTWARDS_WHITE_ARROW = '&#8678'
UPWARDS_WHITE_ARROW = '&#8679'
RIGHTWARDS_WHITE_ARROW = '&#8680'
DOWNWARDS_WHITE_ARROW = '&#8681'
UPWARDS_WHITE_ARROW_FROM_BAR = '&#8682'
UPWARDS_WHITE_ARROW_ON_PEDESTAL = '&#8683'
UPWARDS_WHITE_ARROW_ON_PEDESTAL_WITH_HORIZONTAL_BAR = '&#8684'
UPWARDS_WHITE_ARROW_ON_PEDESTAL_WITH_VERTICAL_BAR = '&#8685'
UPWARDS_WHITE_DOUBLE_ARROW = '&#8686'
UPWARDS_WHITE_DOUBLE_ARROW_ON_PEDESTAL = '&#8687'
RIGHTWARDS_WHITE_ARROW_FROM_WALL = '&#8688'
NORTH_WEST_ARROW_TO_CORNER = '&#8689'
SOUTH_EAST_ARROW_TO_CORNER = '&#8690'
UP_DOWN_WHITE_ARROW = '&#8691'
RIGHT_ARROW_WITH_SMALL_CIRCLE = '&#8692'
DOWNWARDS_ARROW_LEFTWARDS_OF_UPWARDS_ARROW = '&#8693'
THREE_RIGHTWARDS_ARROWS = '&#8694'
LEFTWARDS_ARROW_WITH_VERTICAL_STROKE = '&#8695'
RIGHTWARDS_ARROW_WITH_VERTICAL_STROKE = '&#8696'
LEFT_RIGHT_ARROW_WITH_VERTICAL_STROKE = '&#8697'
LEFTWARDS_ARROW_WITH_DOUBLE_VERTICAL_STROKE = '&#8698'
RIGHTWARDS_ARROW_WITH_DOUBLE_VERTICAL_STROKE = '&#8699'
LEFT_RIGHT_ARROW_WITH_DOUBLE_VERTICAL_STROKE = '&#8700'
LEFTWARDS_OPEN_HEADED_ARROW = '&#8701'
RIGHTWARDS_OPEN_HEADED_ARROW = '&#8702'
LEFT_RIGHT_OPEN_HEADED_ARROW = '&#8703'
