"""
Related Pages:

		https://www.w3schools.com/charsets/ref_html_entities_4.asp
"""

AMPERSAND = '&amp;'
LESS_THAN = '&lt;'
GREATER_THAN = '&gt;'
NO_BREAK_SPACE = '&nbsp;'
NON_BREAKING_SPACE = '&nbsp;'
INVERTED_EXCLAMATION_MARK = '&iexcl;'
CENT_SIGN = '&cent;'
POUND_SIGN = '&pound;'
CURRENCY_SIGN = '&curren;'
YEN_SIGN = '&yen;'
YUAN_SIGN = '&yen;'
BROKEN_BAR = '&brvbar;'
BROKEN_VERTICAL_BAR = '&brvbar;'
SECTION_SIGN = '&sect;'
DIAERESIS = '&uml;'
SPACING_DIAERESIS = '&uml;'
COPYRIGHT_SIGN = '&copy;'
FEMININE_ORDINAL_INDICATOR = '&ordf;'
LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK = '&laquo;'
LEFT_POINTING_GUILLEMET = '&laquo;'
NOT_SIGN = '&not;'
SOFT_HYPHEN = '&shy;'
DISCRETIONARY_HYPHEN = '&shy;'
REGISTERED_SIGN = '&reg;'
REGISTERED_TRADE_MARK_SIGN = '&reg;'
MACRON = '&macr;'
SPACING_MACRON = '&macr;'
OVERLINE = '&macr;'
APL_OVERBAR = '&macr;'
DEGREE_SIGN = '&deg;'
PLUS_MINUS_SIGN = '&plusmn;'
PLUS_OR_MINUS_SIGN = '&plusmn;'
SUPERSCRIPT_TWO = '&sup2;'
SUPERSCRIPT_DIGIT_TWO = '&sup2;'
SQUARED = '&sup2;'
SUPERSCRIPT_THREE = '&sup3;'
SUPERSCRIPT_DIGIT_THREE = '&sup3;'
CUBED = '&sup3;'
ACUTE_ACCENT = '&acute;'
SPACING_ACUTE = '&acute;'
MICRO_SIGN = '&micro;'
PILCROW_SIGN = '&para;'
PARAGRAPH_SIGN = '&para;'
MIDDLE_DOT = '&middot;'
GEORGIAN_COMMA = '&middot;'
GREEK_MIDDLE_DOT = '&middot;'
CEDILLA = '&cedil;'
SPACING_CEDILLA = '&cedil;'
SUPERSCRIPT_ONE = '&sup1;'
SUPERSCRIPT_DIGIT_ONE = '&sup1;'
MASCULINE_ORDINAL_INDICATOR = '&ordm;'
RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK = '&raquo;'
RIGHT_POINTING_GUILLEMET = '&raquo;'
VULGAR_FRACTION_ONE_QUARTER = '&frac14;'
FRACTION_ONE_QUARTER = '&frac14;'
VULGAR_FRACTION_ONE_HALF = '&frac12;'
FRACTION_ONE_HALF = '&frac12;'
VULGAR_FRACTION_THREE_QUARTERS = '&frac34;'
FRACTION_THREE_QUARTERS = '&frac34;'
INVERTED_QUESTION_MARK = '&iquest;'
TURNED_QUESTION_MARK = '&iquest;'
LATIN_CAPITAL_LETTER_A_WITH_GRAVE = '&Agrave;'
LATIN_CAPITAL_LETTER_A_GRAVE = '&Agrave;'
LATIN_CAPITAL_LETTER_A_WITH_ACUTE = '&Aacute;'
LATIN_CAPITAL_LETTER_A_WITH_CIRCUMFLEX = '&Acirc;'
LATIN_CAPITAL_LETTER_A_WITH_TILDE = '&Atilde;'
LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS = '&Auml;'
LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE = '&Aring;'
LATIN_CAPITAL_LETTER_A_RING = '&Aring;'
LATIN_CAPITAL_LETTER_AE = '&AElig;'
LATIN_CAPITAL_LIGATURE_AE = '&AElig;'
LATIN_CAPITAL_LETTER_C_WITH_CEDILLA = '&Ccedil;'
LATIN_CAPITAL_LETTER_E_WITH_GRAVE = '&Egrave;'
LATIN_CAPITAL_LETTER_E_WITH_ACUTE = '&Eacute;'
LATIN_CAPITAL_LETTER_E_WITH_CIRCUMFLEX = '&Ecirc;'
LATIN_CAPITAL_LETTER_E_WITH_DIAERESIS = '&Euml;'
LATIN_CAPITAL_LETTER_I_WITH_GRAVE = '&Igrave;'
LATIN_CAPITAL_LETTER_I_WITH_ACUTE = '&Iacute;'
LATIN_CAPITAL_LETTER_I_WITH_CIRCUMFLEX = '&Icirc;'
LATIN_CAPITAL_LETTER_I_WITH_DIAERESIS = '&Iuml;'
LATIN_CAPITAL_LETTER_ETH = '&ETH;'
LATIN_CAPITAL_LETTER_N_WITH_TILDE = '&Ntilde;'
LATIN_CAPITAL_LETTER_O_WITH_GRAVE = '&Ograve;'
LATIN_CAPITAL_LETTER_O_WITH_ACUTE = '&Oacute;'
LATIN_CAPITAL_LETTER_O_WITH_CIRCUMFLEX = '&Ocirc;'
LATIN_CAPITAL_LETTER_O_WITH_TILDE = '&Otilde;'
LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS = '&Ouml;'
MULTIPLICATION_SIGN = '&times;'
LATIN_CAPITAL_LETTER_O_WITH_STROKE = '&Oslash;'
LATIN_CAPITAL_LETTER_O_SLASH = '&Oslash;'
LATIN_CAPITAL_LETTER_U_WITH_GRAVE = '&Ugrave;'
LATIN_CAPITAL_LETTER_U_WITH_ACUTE = '&Uacute;'
LATIN_CAPITAL_LETTER_U_WITH_CIRCUMFLEX = '&Ucirc;'
LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS = '&Uuml;'
LATIN_CAPITAL_LETTER_Y_WITH_ACUTE = '&Yacute;'
LATIN_CAPITAL_LETTER_THORN = '&THORN;'
LATIN_SMALL_LETTER_SHARP_S = '&szlig;'
ESS_ZED = '&szlig;'
LATIN_SMALL_LETTER_A_WITH_GRAVE = '&agrave;'
LATIN_SMALL_LETTER_A_GRAVE = '&agrave;'
LATIN_SMALL_LETTER_A_WITH_ACUTE = '&aacute;'
LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX = '&acirc;'
LATIN_SMALL_LETTER_A_WITH_TILDE = '&atilde;'
LATIN_SMALL_LETTER_A_WITH_DIAERESIS = '&auml;'
LATIN_SMALL_LETTER_A_WITH_RING_ABOVE = '&aring;'
LATIN_SMALL_LETTER_A_RING = '&aring;'
LATIN_SMALL_LETTER_AE = '&aelig;'
LATIN_SMALL_LIGATURE_AE = '&aelig;'
LATIN_SMALL_LETTER_C_WITH_CEDILLA = '&ccedil;'
LATIN_SMALL_LETTER_E_WITH_GRAVE = '&egrave;'
LATIN_SMALL_LETTER_E_WITH_ACUTE = '&eacute;'
LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX = '&ecirc;'
LATIN_SMALL_LETTER_E_WITH_DIAERESIS = '&euml;'
LATIN_SMALL_LETTER_I_WITH_GRAVE = '&igrave;'
LATIN_SMALL_LETTER_I_WITH_ACUTE = '&iacute;'
LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX = '&icirc;'
LATIN_SMALL_LETTER_I_WITH_DIAERESIS = '&iuml;'
LATIN_SMALL_LETTER_ETH = '&eth;'
LATIN_SMALL_LETTER_N_WITH_TILDE = '&ntilde;'
LATIN_SMALL_LETTER_O_WITH_GRAVE = '&ograve;'
LATIN_SMALL_LETTER_O_WITH_ACUTE = '&oacute;'
LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX = '&ocirc;'
LATIN_SMALL_LETTER_O_WITH_TILDE = '&otilde;'
LATIN_SMALL_LETTER_O_WITH_DIAERESIS = '&ouml;'
DIVISION_SIGN = '&divide;'
LATIN_SMALL_LETTER_O_WITH_STROKE = '&oslash;'
LATIN_SMALL_LETTER_O_SLASH = '&oslash;'
LATIN_SMALL_LETTER_U_WITH_GRAVE = '&ugrave;'
LATIN_SMALL_LETTER_U_WITH_ACUTE = '&uacute;'
LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX = '&ucirc;'
LATIN_SMALL_LETTER_U_WITH_DIAERESIS = '&uuml;'
LATIN_SMALL_LETTER_Y_WITH_ACUTE = '&yacute;'
LATIN_SMALL_LETTER_THORN = '&thorn;'
LATIN_SMALL_LETTER_Y_WITH_DIAERESIS = '&yuml;'
LATIN_SMALL_F_WITH_HOOK = '&fnof;'
FUNCTION = '&fnof;'
FLORIN = '&fnof;'
GREEK_CAPITAL_LETTER_ALPHA = '&Alpha;'
GREEK_CAPITAL_LETTER_BETA = '&Beta;'
GREEK_CAPITAL_LETTER_GAMMA = '&Gamma;'
GREEK_CAPITAL_LETTER_DELTA = '&Delta;'
GREEK_CAPITAL_LETTER_EPSILON = '&Epsilon;'
GREEK_CAPITAL_LETTER_ZETA = '&Zeta;'
GREEK_CAPITAL_LETTER_ETA = '&Eta;'
GREEK_CAPITAL_LETTER_THETA = '&Theta;'
GREEK_CAPITAL_LETTER_IOTA = '&Iota;'
GREEK_CAPITAL_LETTER_KAPPA = '&Kappa;'
GREEK_CAPITAL_LETTER_LAMBDA = '&Lambda;'
GREEK_CAPITAL_LETTER_MU = '&Mu;'
GREEK_CAPITAL_LETTER_NU = '&Nu;'
GREEK_CAPITAL_LETTER_XI = '&Xi;'
GREEK_CAPITAL_LETTER_OMICRON = '&Omicron;'
GREEK_CAPITAL_LETTER_PI = '&Pi;'
GREEK_CAPITAL_LETTER_RHO = '&Rho;'
GREEK_CAPITAL_LETTER_SIGMA = '&Sigma;'
GREEK_CAPITAL_LETTER_TAU = '&Tau;'
GREEK_CAPITAL_LETTER_UPSILON = '&Upsilon;'
GREEK_CAPITAL_LETTER_PHI = '&Phi;'
GREEK_CAPITAL_LETTER_CHI = '&Chi;'
GREEK_CAPITAL_LETTER_PSI = '&Psi;'
GREEK_CAPITAL_LETTER_OMEGA = '&Omega;'
GREEK_SMAL_LETTER_ALPHA = '&alpha;'
GREEK_SMAL_LETTER_BETA = '&beta;'
GREEK_SMAL_LETTER_GAMMA = '&gamma;'
GREEK_SMAL_LETTER_DELTA = '&delta;'
GREEK_SMAL_LETTER_EPSILON = '&epsilon;'
GREEK_SMAL_LETTER_ZETA = '&zeta;'
GREEK_SMAL_LETTER_ETA = '&eta;'
GREEK_SMAL_LETTER_THETA = '&theta;'
GREEK_SMAL_LETTER_IOTA = '&iota;'
GREEK_SMAL_LETTER_KAPPA = '&kappa;'
GREEK_SMAL_LETTER_LAMBDA = '&lambda;'
GREEK_SMAL_LETTER_MU = '&mu;'
GREEK_SMAL_LETTER_NU = '&nu;'
GREEK_SMAL_LETTER_XI = '&xi;'
GREEK_SMAL_LETTER_OMICRON = '&omicron;'
GREEK_SMAL_LETTER_PI = '&pi;'
GREEK_SMAL_LETTER_RHO = '&rho;'
GREEK_SMAL_LETTER_FINAL_SIGMA = '&sigmaf;'
GREEK_SMAL_LETTER_SIGMA = '&sigma;'
GREEK_SMAL_LETTER_TAU = '&tau;'
GREEK_SMAL_LETTER_UPSILON = '&upsilon;'
GREEK_SMAL_LETTER_PHI = '&phi;'
GREEK_SMAL_LETTER_CHI = '&chi;'
GREEK_SMAL_LETTER_PSI = '&psi;'
GREEK_SMAL_LETTER_OMEGA = '&omega;'
GREEK_SMAL_LETTER_THETA_SYMBOL = '&thetasym;'
GREEK_UPSILON_WITH_HOOK_SYMBOL = '&upsih;'
GREEK_PI_SYMBOL = '&piv;'
