"""
Related Pages:

		https://www.w3schools.com/icons/fontawesome_icons_webapp.asp
"""

ICON_ADDRESS_BOOK = 'fa fa-address-book'
ICON_ADDRESS_BOOK_O = 'fa fa-address-book-o'
ICON_ADDRESS_CARD = 'fa fa-address-card'
ICON_ADDRESS_CARD_O = 'fa fa-address-card-o'
ICON_ADJUST = 'fa fa-adjust'
ICON_AMERICAN_SIGN_LANGUAGE_INTERPRETING = 'fa fa-american-sign-language-interpreting'
ICON_ANCHOR = 'fa fa-anchor'
ICON_ARCHIVE = 'fa fa-archive'
ICON_AREA_CHART = 'fa fa-area-chart'
ICON_ARROWS = 'fa fa-arrows'
ICON_ARROWS_H = 'fa fa-arrows-h'
ICON_ARROWS_V = 'fa fa-arrows-v'
ICON_ASL_INTERPRETING = 'fa fa-asl-interpreting'
ICON_ASSISTIVE_LISTENING_SYSTEMS = 'fa fa-assistive-listening-systems'
ICON_ASTERISK = 'fa fa-asterisk'
ICON_AT = 'fa fa-at'
ICON_AUTOMOBILE = 'fa fa-automobile'
ICON_AUDIO_DESCRIPTION = 'fa fa-audio-description'
ICON_BALANCE_SCALE = 'fa fa-balance-scale'
ICON_BAN = 'fa fa-ban'
ICON_BANK = 'fa fa-bank'
ICON_BAR_CHART = 'fa fa-bar-chart'
ICON_BAR_CHART_O = 'fa fa-bar-chart-o'
ICON_BARCODE = 'fa fa-barcode'
ICON_BARS = 'fa fa-bars'
ICON_BATH = 'fa fa-bath'
ICON_BATHTUB = 'fa fa-bathtub'
ICON_BATTERY_0 = 'fa fa-battery-0'
ICON_BATTERY_1 = 'fa fa-battery-1'
ICON_BATTERY_2 = 'fa fa-battery-2'
ICON_BATTERY_3 = 'fa fa-battery-3'
ICON_BATTERY_4 = 'fa fa-battery-4'
ICON_BATTERY_EMPTY = 'fa fa-battery-empty'
ICON_BATTERY_FULL = 'fa fa-battery-full'
ICON_BATTERY_HALF = 'fa fa-battery-half'
ICON_BATTERY_QUARTER = 'fa fa-battery-quarter'
ICON_BATTERY_THREE_QUARTERS = 'fa fa-battery-three-quarters'
ICON_BED = 'fa fa-bed'
ICON_BEER = 'fa fa-beer'
ICON_BELL = 'fa fa-bell'
ICON_BELL_O = 'fa fa-bell-o'
ICON_BELL_SLASH = 'fa fa-bell-slash'
ICON_BELL_SLASH_O = 'fa fa-bell-slash-o'
ICON_BICYCLE = 'fa fa-bicycle'
ICON_BINOCULARS = 'fa fa-binoculars'
ICON_BIRTHDAY_CAKE = 'fa fa-birthday-cake'
ICON_BLIND = 'fa fa-blind'
ICON_BOLT = 'fa fa-bolt'
ICON_BOMB = 'fa fa-bomb'
ICON_BOOK = 'fa fa-book'
ICON_BOOKMARK = 'fa fa-bookmark'
ICON_BOOKMARK_O = 'fa fa-bookmark-o'
ICON_BRAILLE = 'fa fa-braille'
ICON_BRIEFCASE = 'fa fa-briefcase'
ICON_BUG = 'fa fa-bug'
ICON_BUILDING = 'fa fa-building'
ICON_BUILDING_O = 'fa fa-building-o'
ICON_BULLHORN = 'fa fa-bullhorn'
ICON_BULLSEYE = 'fa fa-bullseye'
ICON_BUS = 'fa fa-bus'
ICON_CAB = 'fa fa-cab'
ICON_CALCULATOR = 'fa fa-calculator'
ICON_CALENDAR = 'fa fa-calendar'
ICON_CALENDAR_O = 'fa fa-calendar-o'
ICON_CALENDAR_CHECK_O = 'fa fa-calendar-check-o'
ICON_CALENDAR_MINUS_O = 'fa fa-calendar-minus-o'
ICON_CALENDAR_PLUS_O = 'fa fa-calendar-plus-o'
ICON_CALENDAR_TIMES_O = 'fa fa-calendar-times-o'
ICON_CAMERA = 'fa fa-camera'
ICON_CAMERA_RETRO = 'fa fa-camera-retro'
ICON_CAR = 'fa fa-car'
ICON_CARET_SQUARE_O_DOWN = 'fa fa-caret-square-o-down'
ICON_CARET_SQUARE_O_LEFT = 'fa fa-caret-square-o-left'
ICON_CARET_SQUARE_O_RIGHT = 'fa fa-caret-square-o-right'
ICON_CARET_SQUARE_O_UP = 'fa fa-caret-square-o-up'
ICON_CART_ARROW_DOWN = 'fa fa-cart-arrow-down'
ICON_CART_PLUS = 'fa fa-cart-plus'
ICON_CC = 'fa fa-cc'
ICON_CERTIFICATE = 'fa fa-certificate'
ICON_CHECK = 'fa fa-check'
ICON_CHECK_CIRCLE = 'fa fa-check-circle'
ICON_CHECK_CIRCLE_O = 'fa fa-check-circle-o'
ICON_CHECK_SQUARE = 'fa fa-check-square'
ICON_CHECK_SQUARE_O = 'fa fa-check-square-o'
ICON_CHILD = 'fa fa-child'
ICON_CIRCLE = 'fa fa-circle'
ICON_CIRCLE_O = 'fa fa-circle-o'
ICON_CIRCLE_O_NOTCH = 'fa fa-circle-o-notch'
ICON_CIRCLE_THIN = 'fa fa-circle-thin'
ICON_CLOCK_O = 'fa fa-clock-o'
ICON_CLONE = 'fa fa-clone'
ICON_CLOSE = 'fa fa-close'
ICON_CLOUD = 'fa fa-cloud'
ICON_CLOUD_DOWNLOAD = 'fa fa-cloud-download'
ICON_CLOUD_UPLOAD = 'fa fa-cloud-upload'
ICON_CODE = 'fa fa-code'
ICON_CODE_FORK = 'fa fa-code-fork'
ICON_COFFEE = 'fa fa-coffee'
ICON_COG = 'fa fa-cog'
ICON_COGS = 'fa fa-cogs'
ICON_COMMENT = 'fa fa-comment'
ICON_COMMENT_O = 'fa fa-comment-o'
ICON_COMMENTS = 'fa fa-comments'
ICON_COMMENTS_O = 'fa fa-comments-o'
ICON_COMMENTING = 'fa fa-commenting'
ICON_COMMENTING_O = 'fa fa-commenting-o'
ICON_COMPASS = 'fa fa-compass'
ICON_COPYRIGHT = 'fa fa-copyright'
ICON_CREDIT_CARD = 'fa fa-credit-card'
ICON_CREDIT_CARD_ALT = 'fa fa-credit-card-alt'
ICON_CREATIVE_COMMONS = 'fa fa-creative-commons'
ICON_CROP = 'fa fa-crop'
ICON_CROSSHAIRS = 'fa fa-crosshairs'
ICON_CUBE = 'fa fa-cube'
ICON_CUBES = 'fa fa-cubes'
ICON_CUTLERY = 'fa fa-cutlery'
ICON_DASHBOARD = 'fa fa-dashboard'
ICON_DATABASE = 'fa fa-database'
ICON_DEAF = 'fa fa-deaf'
ICON_DEAFNESS = 'fa fa-deafness'
ICON_DESKTOP = 'fa fa-desktop'
ICON_DIAMOND = 'fa fa-diamond'
ICON_DOT_CIRCLE_O = 'fa fa-dot-circle-o'
ICON_DOWNLOAD = 'fa fa-download'
ICON_DRIVERS_LICENSE = 'fa fa-drivers-license'
ICON_DRIVERS_LICENSE_O = 'fa fa-drivers-license-o'
ICON_EDIT = 'fa fa-edit'
ICON_ELLIPSIS_H = 'fa fa-ellipsis-h'
ICON_ELLIPSIS_V = 'fa fa-ellipsis-v'
ICON_ENVELOPE = 'fa fa-envelope'
ICON_ENVELOPE_O = 'fa fa-envelope-o'
ICON_ENVELOPE_OPEN = 'fa fa-envelope-open'
ICON_ENVELOPE_OPEN_O = 'fa fa-envelope-open-o'
ICON_ENVELOPE_SQUARE = 'fa fa-envelope-square'
ICON_ERASER = 'fa fa-eraser'
ICON_EXCHANGE = 'fa fa-exchange'
ICON_EXCLAMATION = 'fa fa-exclamation'
ICON_EXCLAMATION_CIRCLE = 'fa fa-exclamation-circle'
ICON_EXCLAMATION_TRIANGLE = 'fa fa-exclamation-triangle'
ICON_EXTERNAL_LINK = 'fa fa-external-link'
ICON_EXTERNAL_LINK_SQUARE = 'fa fa-external-link-square'
ICON_EYE = 'fa fa-eye'
ICON_EYE_SLASH = 'fa fa-eye-slash'
ICON_EYEDROPPER = 'fa fa-eyedropper'
ICON_FAX = 'fa fa-fax'
ICON_FEMALE = 'fa fa-female'
ICON_FIGHTER_JET = 'fa fa-fighter-jet'
ICON_FILE_ARCHIVE_O = 'fa fa-file-archive-o'
ICON_FILE_AUDIO_O = 'fa fa-file-audio-o'
ICON_FILE_CODE_O = 'fa fa-file-code-o'
ICON_FILE_EXCEL_O = 'fa fa-file-excel-o'
ICON_FILE_IMAGE_O = 'fa fa-file-image-o'
ICON_FILE_MOVIE_O = 'fa fa-file-movie-o'
ICON_FILE_PDF_O = 'fa fa-file-pdf-o'
ICON_FILE_PHOTO_O = 'fa fa-file-photo-o'
ICON_FILE_PICTURE_O = 'fa fa-file-picture-o'
ICON_FILE_POWERPOINT_O = 'fa fa-file-powerpoint-o'
ICON_FILE_SOUND_O = 'fa fa-file-sound-o'
ICON_FILE_VIDEO_O = 'fa fa-file-video-o'
ICON_FILE_WORD_O = 'fa fa-file-word-o'
ICON_FILE_ZIP_O = 'fa fa-file-zip-o'
ICON_FILM = 'fa fa-film'
ICON_FILTER = 'fa fa-filter'
ICON_FIRE = 'fa fa-fire'
ICON_FIRE_EXTINGUISHER = 'fa fa-fire-extinguisher'
ICON_FLAG = 'fa fa-flag'
ICON_FLAG_CHECKERED = 'fa fa-flag-checkered'
ICON_FLAG_O = 'fa fa-flag-o'
ICON_FLASH = 'fa fa-flash'
ICON_FLASK = 'fa fa-flask'
ICON_FOLDER = 'fa fa-folder'
ICON_FOLDER_O = 'fa fa-folder-o'
ICON_FOLDER_OPEN = 'fa fa-folder-open'
ICON_FOLDER_OPEN_O = 'fa fa-folder-open-o'
ICON_FROWN_O = 'fa fa-frown-o'
ICON_FUTBOL_O = 'fa fa-futbol-o'
ICON_GAMEPAD = 'fa fa-gamepad'
ICON_GAVEL = 'fa fa-gavel'
ICON_GEAR = 'fa fa-gear'
ICON_GEARS = 'fa fa-gears'
ICON_GENDERLESS = 'fa fa-genderless'
ICON_GIFT = 'fa fa-gift'
ICON_GLASS = 'fa fa-glass'
ICON_GLOBE = 'fa fa-globe'
ICON_GRADUATION_CAP = 'fa fa-graduation-cap'
ICON_GROUP = 'fa fa-group'
ICON_HARD_OF_HEARING = 'fa fa-hard-of-hearing'
ICON_HDD_O = 'fa fa-hdd-o'
ICON_HANDSHAKE_O = 'fa fa-handshake-o'
ICON_HASHTAG = 'fa fa-hashtag'
ICON_HEADPHONES = 'fa fa-headphones'
ICON_HEART = 'fa fa-heart'
ICON_HEART_O = 'fa fa-heart-o'
ICON_HEARTBEAT = 'fa fa-heartbeat'
ICON_HISTORY = 'fa fa-history'
ICON_HOME = 'fa fa-home'
ICON_HOTEL = 'fa fa-hotel'
ICON_HOURGLASS = 'fa fa-hourglass'
ICON_HOURGLASS_1 = 'fa fa-hourglass-1'
ICON_HOURGLASS_2 = 'fa fa-hourglass-2'
ICON_HOURGLASS_3 = 'fa fa-hourglass-3'
ICON_HOURGLASS_END = 'fa fa-hourglass-end'
ICON_HOURGLASS_HALF = 'fa fa-hourglass-half'
ICON_HOURGLASS_O = 'fa fa-hourglass-o'
ICON_HOURGLASS_START = 'fa fa-hourglass-start'
ICON_I_CURSOR = 'fa fa-i-cursor'
ICON_ID_BADGE = 'fa fa-id-badge'
ICON_ID_CARD = 'fa fa-id-card'
ICON_ID_CARD_O = 'fa fa-id-card-o'
ICON_IMAGE = 'fa fa-image'
ICON_INBOX = 'fa fa-inbox'
ICON_INDUSTRY = 'fa fa-industry'
ICON_INFO = 'fa fa-info'
ICON_INFO_CIRCLE = 'fa fa-info-circle'
ICON_INSTITUTION = 'fa fa-institution'
ICON_KEY = 'fa fa-key'
ICON_KEYBOARD_O = 'fa fa-keyboard-o'
ICON_LANGUAGE = 'fa fa-language'
ICON_LAPTOP = 'fa fa-laptop'
ICON_LEAF = 'fa fa-leaf'
ICON_LEGAL = 'fa fa-legal'
ICON_LEMON_O = 'fa fa-lemon-o'
ICON_LEVEL_DOWN = 'fa fa-level-down'
ICON_LEVEL_UP = 'fa fa-level-up'
ICON_LIFE_BOUY = 'fa fa-life-bouy'
ICON_LIFE_BUOY = 'fa fa-life-buoy'
ICON_LIFE_RING = 'fa fa-life-ring'
ICON_LIFE_SAVER = 'fa fa-life-saver'
ICON_LIGHTBULB_O = 'fa fa-lightbulb-o'
ICON_LINE_CHART = 'fa fa-line-chart'
ICON_LOCATION_ARROW = 'fa fa-location-arrow'
ICON_LOCK = 'fa fa-lock'
ICON_LOW_VISION = 'fa fa-low-vision'
ICON_MAGIC = 'fa fa-magic'
ICON_MAGNET = 'fa fa-magnet'
ICON_MAIL_FORWARD = 'fa fa-mail-forward'
ICON_MAIL_REPLY = 'fa fa-mail-reply'
ICON_MAIL_REPLY_ALL = 'fa fa-mail-reply-all'
ICON_MALE = 'fa fa-male'
ICON_MAP = 'fa fa-map'
ICON_MAP_O = 'fa fa-map-o'
ICON_MAP_PIN = 'fa fa-map-pin'
ICON_MAP_SIGNS = 'fa fa-map-signs'
ICON_MAP_MARKER = 'fa fa-map-marker'
ICON_MEH_O = 'fa fa-meh-o'
ICON_MICROCHIP = 'fa fa-microchip'
ICON_MICROPHONE = 'fa fa-microphone'
ICON_MICROPHONE_SLASH = 'fa fa-microphone-slash'
ICON_MINUS = 'fa fa-minus'
ICON_MINUS_CIRCLE = 'fa fa-minus-circle'
ICON_MINUS_SQUARE = 'fa fa-minus-square'
ICON_MINUS_SQUARE_O = 'fa fa-minus-square-o'
ICON_MOBILE = 'fa fa-mobile'
ICON_MOBILE_PHONE = 'fa fa-mobile-phone'
ICON_MONEY = 'fa fa-money'
ICON_MOON_O = 'fa fa-moon-o'
ICON_MORTAR_BOARD = 'fa fa-mortar-board'
ICON_MOTORCYCLE = 'fa fa-motorcycle'
ICON_MOUSE_POINTER = 'fa fa-mouse-pointer'
ICON_MUSIC = 'fa fa-music'
ICON_NAVICON = 'fa fa-navicon'
ICON_NEWSPAPER_O = 'fa fa-newspaper-o'
ICON_OBJECT_GROUP = 'fa fa-object-group'
ICON_OBJECT_UNGROUP = 'fa fa-object-ungroup'
ICON_PAINT_BRUSH = 'fa fa-paint-brush'
ICON_PAPER_PLANE = 'fa fa-paper-plane'
ICON_PAPER_PLANE_O = 'fa fa-paper-plane-o'
ICON_PAW = 'fa fa-paw'
ICON_PENCIL = 'fa fa-pencil'
ICON_PENCIL_SQUARE = 'fa fa-pencil-square'
ICON_PENCIL_SQUARE_O = 'fa fa-pencil-square-o'
ICON_PERCENT = 'fa fa-percent'
ICON_PHONE = 'fa fa-phone'
ICON_PHONE_SQUARE = 'fa fa-phone-square'
ICON_PHOTO = 'fa fa-photo'
ICON_PICTURE_O = 'fa fa-picture-o'
ICON_PIE_CHART = 'fa fa-pie-chart'
ICON_PLANE = 'fa fa-plane'
ICON_PLUG = 'fa fa-plug'
ICON_PLUS = 'fa fa-plus'
ICON_PLUS_CIRCLE = 'fa fa-plus-circle'
ICON_PLUS_SQUARE = 'fa fa-plus-square'
ICON_PLUS_SQUARE_O = 'fa fa-plus-square-o'
ICON_PODCAST = 'fa fa-podcast'
ICON_POWER_OFF = 'fa fa-power-off'
ICON_PRINT = 'fa fa-print'
ICON_PUZZLE_PIECE = 'fa fa-puzzle-piece'
ICON_QRCODE = 'fa fa-qrcode'
ICON_QUESTION = 'fa fa-question'
ICON_QUESTION_CIRCLE = 'fa fa-question-circle'
ICON_QUESTION_CIRCLE_O = 'fa fa-question-circle-o'
ICON_QUOTE_LEFT = 'fa fa-quote-left'
ICON_QUOTE_RIGHT = 'fa fa-quote-right'
ICON_RANDOM = 'fa fa-random'
ICON_RECYCLE = 'fa fa-recycle'
ICON_REFRESH = 'fa fa-refresh'
ICON_REGISTERED = 'fa fa-registered'
ICON_REMOVE = 'fa fa-remove'
ICON_REORDER = 'fa fa-reorder'
ICON_REPLY = 'fa fa-reply'
ICON_REPLY_ALL = 'fa fa-reply-all'
ICON_RETWEET = 'fa fa-retweet'
ICON_ROAD = 'fa fa-road'
ICON_ROCKET = 'fa fa-rocket'
ICON_RSS = 'fa fa-rss'
ICON_RSS_SQUARE = 'fa fa-rss-square'
ICON_S15 = 'fa fa-s15'
ICON_SEARCH = 'fa fa-search'
ICON_SEARCH_MINUS = 'fa fa-search-minus'
ICON_SEARCH_PLUS = 'fa fa-search-plus'
ICON_SEND = 'fa fa-send'
ICON_SEND_O = 'fa fa-send-o'
ICON_SERVER = 'fa fa-server'
ICON_SHARE = 'fa fa-share'
ICON_SHARE_ALT = 'fa fa-share-alt'
ICON_SHARE_ALT_SQUARE = 'fa fa-share-alt-square'
ICON_SHARE_SQUARE = 'fa fa-share-square'
ICON_SHARE_SQUARE_O = 'fa fa-share-square-o'
ICON_SHIELD = 'fa fa-shield'
ICON_SHIP = 'fa fa-ship'
ICON_SHOPPING_BAG = 'fa fa-shopping-bag'
ICON_SHOPPING_BASKET = 'fa fa-shopping-basket'
ICON_SHOPPING_CART = 'fa fa-shopping-cart'
ICON_SHOWER = 'fa fa-shower'
ICON_SIGN_IN = 'fa fa-sign-in'
ICON_SIGN_OUT = 'fa fa-sign-out'
ICON_SIGN_LANGUAGE = 'fa fa-sign-language'
ICON_SIGNAL = 'fa fa-signal'
ICON_SIGNING = 'fa fa-signing'
ICON_SITEMAP = 'fa fa-sitemap'
ICON_SLIDERS = 'fa fa-sliders'
ICON_SMILE_O = 'fa fa-smile-o'
ICON_SNOWFLAKE_O = 'fa fa-snowflake-o'
ICON_SOCCER_BALL_O = 'fa fa-soccer-ball-o'
ICON_SORT = 'fa fa-sort'
ICON_SORT_ALPHA_ASC = 'fa fa-sort-alpha-asc'
ICON_SORT_ALPHA_DESC = 'fa fa-sort-alpha-desc'
ICON_SORT_AMOUNT_ASC = 'fa fa-sort-amount-asc'
ICON_SORT_AMOUNT_DESC = 'fa fa-sort-amount-desc'
ICON_SORT_ASC = 'fa fa-sort-asc'
ICON_SORT_DESC = 'fa fa-sort-desc'
ICON_SORT_DOWN = 'fa fa-sort-down'
ICON_SORT_NUMERIC_ASC = 'fa fa-sort-numeric-asc'
ICON_SORT_NUMERIC_DESC = 'fa fa-sort-numeric-desc'
ICON_SORT_UP = 'fa fa-sort-up'
ICON_SPACE_SHUTTLE = 'fa fa-space-shuttle'
ICON_SPINNER = 'fa fa-spinner'
ICON_SPOON = 'fa fa-spoon'
ICON_SQUARE = 'fa fa-square'
ICON_SQUARE_O = 'fa fa-square-o'
ICON_STAR = 'fa fa-star'
ICON_STAR_HALF = 'fa fa-star-half'
ICON_STAR_HALF_EMPTY = 'fa fa-star-half-empty'
ICON_STAR_HALF_FULL = 'fa fa-star-half-full'
ICON_STAR_HALF_O = 'fa fa-star-half-o'
ICON_STAR_O = 'fa fa-star-o'
ICON_STICKY_NOTE = 'fa fa-sticky-note'
ICON_STICKY_NOTE_O = 'fa fa-sticky-note-o'
ICON_STREET_VIEW = 'fa fa-street-view'
ICON_SUITCASE = 'fa fa-suitcase'
ICON_SUN_O = 'fa fa-sun-o'
ICON_SUPPORT = 'fa fa-support'
ICON_TABLET = 'fa fa-tablet'
ICON_TACHOMETER = 'fa fa-tachometer'
ICON_TAG = 'fa fa-tag'
ICON_TAGS = 'fa fa-tags'
ICON_TASKS = 'fa fa-tasks'
ICON_TAXI = 'fa fa-taxi'
ICON_TELEVISION = 'fa fa-television'
ICON_TERMINAL = 'fa fa-terminal'
ICON_THERMOMETER = 'fa fa-thermometer'
ICON_THERMOMETER_0 = 'fa fa-thermometer-0'
ICON_THERMOMETER_1 = 'fa fa-thermometer-1'
ICON_THERMOMETER_2 = 'fa fa-thermometer-2'
ICON_THERMOMETER_3 = 'fa fa-thermometer-3'
ICON_THERMOMETER_4 = 'fa fa-thermometer-4'
ICON_THERMOMETER_EMPTY = 'fa fa-thermometer-empty'
ICON_THERMOMETER_FULL = 'fa fa-thermometer-full'
ICON_THERMOMETER_HALF = 'fa fa-thermometer-half'
ICON_THERMOMETER_QUARTER = 'fa fa-thermometer-quarter'
ICON_THERMOMETER_THREE_QUARTERS = 'fa fa-thermometer-three-quarters'
ICON_THUMB_TACK = 'fa fa-thumb-tack'
ICON_THUMBS_DOWN = 'fa fa-thumbs-down'
ICON_THUMBS_O_UP = 'fa fa-thumbs-o-up'
ICON_THUMBS_UP = 'fa fa-thumbs-up'
ICON_TICKET = 'fa fa-ticket'
ICON_TIMES = 'fa fa-times'
ICON_TIMES_CIRCLE = 'fa fa-times-circle'
ICON_TIMES_CIRCLE_O = 'fa fa-times-circle-o'
ICON_TIMES_RECTANGLE = 'fa fa-times-rectangle'
ICON_TIMES_RECTANGLE_O = 'fa fa-times-rectangle-o'
ICON_TINT = 'fa fa-tint'
ICON_TOGGLE_DOWN = 'fa fa-toggle-down'
ICON_TOGGLE_LEFT = 'fa fa-toggle-left'
ICON_TOGGLE_RIGHT = 'fa fa-toggle-right'
ICON_TOGGLE_UP = 'fa fa-toggle-up'
ICON_TOGGLE_OFF = 'fa fa-toggle-off'
ICON_TOGGLE_ON = 'fa fa-toggle-on'
ICON_TRADEMARK = 'fa fa-trademark'
ICON_TRASH = 'fa fa-trash'
ICON_TRASH_O = 'fa fa-trash-o'
ICON_TREE = 'fa fa-tree'
ICON_TROPHY = 'fa fa-trophy'
ICON_TRUCK = 'fa fa-truck'
ICON_TTY = 'fa fa-tty'
ICON_TV = 'fa fa-tv'
ICON_UMBRELLA = 'fa fa-umbrella'
ICON_UNIVERSAL_ACCESS = 'fa fa-universal-access'
ICON_UNIVERSITY = 'fa fa-university'
ICON_UNLOCK = 'fa fa-unlock'
ICON_UNLOCK_ALT = 'fa fa-unlock-alt'
ICON_UNSORTED = 'fa fa-unsorted'
ICON_UPLOAD = 'fa fa-upload'
ICON_USER = 'fa fa-user'
ICON_USER_CIRCLE = 'fa fa-user-circle'
ICON_USER_CIRCLE_O = 'fa fa-user-circle-o'
ICON_USER_O = 'fa fa-user-o'
ICON_USER_PLUS = 'fa fa-user-plus'
ICON_USER_SECRET = 'fa fa-user-secret'
ICON_USER_TIMES = 'fa fa-user-times'
ICON_USERS = 'fa fa-users'
ICON_VCARD = 'fa fa-vcard'
ICON_VCARD_O = 'fa fa-vcard-o'
ICON_VIDEO_CAMERA = 'fa fa-video-camera'
ICON_VOLUME_CONTROL_PHONE = 'fa fa-volume-control-phone'
ICON_VOLUME_DOWN = 'fa fa-volume-down'
ICON_VOLUME_OFF = 'fa fa-volume-off'
ICON_VOLUME_UP = 'fa fa-volume-up'
ICON_WARNING = 'fa fa-warning'
ICON_WHEELCHAIR = 'fa fa-wheelchair'
ICON_WHEELCHAIR_ALT = 'fa fa-wheelchair-alt'
ICON_WINDOW_CLOSE = 'fa fa-window-close'
ICON_WINDOW_CLOSE_O = 'fa fa-window-close-o'
ICON_WINDOW_MAXIMIZE = 'fa fa-window-maximize'
ICON_WINDOW_MINIMIZE = 'fa fa-window-minimize'
ICON_WINDOW_RESTORE = 'fa fa-window-restore'
ICON_WIFI = 'fa fa-wifi'
ICON_WRENCH = 'fa fa-wrench'

EFFECT_SPIN = 'fa-spin'
EFFECT_PULSE = 'fa-pulse'


if __name__ == '__main__':
  data = '''
  '''
  for rec in data.split("\n"):
    if rec.strip() != "":
      srec = rec.strip().split(" ")
      icon = srec[1].replace("fa-", "ICON_").replace("-", "_")
      print("%s = '%s'" % (icon.upper(), rec.strip()))
