"""
Related Pages:

		https://www.w3schools.com/charsets/ref_utf_punctuation.asp
"""

EN_QUAD = '&#8192'
EM_QUAD = '&#8193'
EN_SPACE = '&#8194'
EM_SPACE = '&#8195'
THREE_PER_EM_SPACE = '&#8196'
FOUR_PER_EM_SPACE = '&#8197'
SIX_PER_EM_SPACE = '&#8198'
FIGURE_SPACE = '&#8199'
PUNCTUATION_SPACE = '&#8200'
THIN_SPACE = '&#8201'
HAIR_SPACE = '&#8202'
ZERO_WIDTH_SPACE = '&#8203'
ZERO_WIDTH_NON_JOINER = '&#8204'
ZERO_WIDTH_JOINER = '&#8205'
LEFT_TO_RIGHT_MARK = '&#8206'
RIGHT_TO_LEFT_MARK = '&#8207'
HYPHEN = '&#8208'
NON_BREAKING_HYPHEN = '&#8209'
FIGURE_DASH = '&#8210'
EN_DASH = '&#8211'
EM_DASH = '&#8212'
HORIZONTAL_BAR = '&#8213'
DOUBLE_VERTICAL_LINE = '&#8214'
DOUBLE_LOW_LINE = '&#8215'
LEFT_SINGLE_QUOTATION_MARK = '&#8216'
RIGHT_SINGLE_QUOTATION_MARK = '&#8217'
SINGLE_LOW_9_QUOTATION_MARK = '&#8218'
SINGLE_HIGH_REVERSED_9_QUOTATION_MARK = '&#8219'
LEFT_DOUBLE_QUOTATION_MARK = '&#8220'
RIGHT_DOUBLE_QUOTATION_MARK = '&#8221'
DOUBLE_LOW_9_QUOTATION_MARK = '&#8222'
DOUBLE_HIGH_REVERSED_9_QUOTATION_MARK = '&#8223'
DAGGER = '&#8224'
DOUBLE_DAGGER = '&#8225'
BULLET = '&#8226'
TRIANGULAR_BULLET = '&#8227'
ONE_DOT_LEADER = '&#8228'
TWO_DOT_LEADER = '&#8229'
HORIZONTAL_ELLIPSIS = '&#8230'
HYPHENATION_POINT = '&#8231'
LINE_SEPARATOR = '&#8232'
PARAGRAPH_SEPARATOR = '&#8233'
LEFT_TO_RIGHT_EMBEDDING = '&#8234'
RIGHT_TO_LEFT_EMBEDDING = '&#8235'
POP_DIRECTIONAL_FORMATTING = '&#8236'
LEFT_TO_RIGHT_OVERRIDE = '&#8237'
RIGHT_TO_LEFT_OVERRIDE = '&#8238'
NARROW_NON_BREAK_SPACE = '&#8239'
PER_MILLE_SIGN = '&#8240'
PER_TEN_THOUSAND_SIGN = '&#8241'
PRIME = '&#8242'
DOUBLE_PRIME = '&#8243'
TRIPLE_PRIME = '&#8244'
REVERSED_PRIME = '&#8245'
REVERSED_DOUBLE_PRIME = '&#8246'
REVERSED_TRIPLE_PRIME = '&#8247'
CARET = '&#8248'
SINGLE_LEFT_POINTING_ANGLE_QUOTATION_MARK = '&#8249'
SINGLE_RIGHT_POINTING_ANGLE_QUOTATION_MARK = '&#8250'
REFERENCE_MARK = '&#8251'
DOUBLE_EXCLAMATION_MARK = '&#8252'
INTERROBANG = '&#8253'
OVERLINE = '&#8254'
UNDERTIE = '&#8255'
CHARACTER_TIE = '&#8256'
CARET_INSERTION_POINT = '&#8257'
ASTERISM = '&#8258'
HYPHEN_BULLET = '&#8259'
FRACTION_SLASH = '&#8260'
LEFT_SQUARE_BRACKET_WITH_QUILL = '&#8261'
RIGHT_SQUARE_BRACKET_WITH_QUILL = '&#8262'
DOUBLE_QUESTION_MARK = '&#8263'
QUESTION_EXCLAMATION_MARK = '&#8264'
EXCLAMATION_QUESTION_MARK = '&#8265'
TIRONIAN_SIGN_ET = '&#8266'
REVERSED_PILCROW_SIGN = '&#8267'
BLACK_LEFTWARDS_BULLET = '&#8268'
BLACK_RIGHTWARDS_BULLET = '&#8269'
LOW_ASTERISK = '&#8270'
REVERSED_SEMICOLON = '&#8271'
CLOSE_UP = '&#8272'
TWO_ASTERISKS_ALIGNED_VERTICALLY = '&#8273'
COMMERCIAL_MINUS_SIGN = '&#8274'
SWUNG_DASH = '&#8275'
INVERTED_UNDERTIE = '&#8276'
FLOWER_PUNCTUATION_MARK = '&#8277'
THREE_DOT_PUNCTUATION = '&#8278'
QUADRUPLE_PRIME = '&#8279'
FOUR_DOT_PUNCTUATION = '&#8280'
FIVE_DOT_PUNCTUATION = '&#8281'
TWO_DOT_PUNCTUATION = '&#8282'
FOUR_DOT_MARK = '&#8283'
DOTTED_CROSS = '&#8284'
TRICOLON = '&#8285'
VERTICAL_FOUR_DOTS = '&#8286'
MEDIUM_MATHEMATICAL_SPACE = '&#8287'
WORD_JOINER = '&#8288'
FUNCTION_APPLICATION = '&#8289'
INVISIBLE_TIMES = '&#8290'
INVISIBLE_SEPARATOR = '&#8291'
INVISIBLE_PLUS = '&#8292'
LEFT_TO_RIGHT_ISOLATE = '&#8294'
RIGHT_TO_LEFT_ISOLATE = '&#8295'
FIRST_STRONG_ISOLATE = '&#8296'
POP_DIRECTIONAL_ISOLATE = '&#8297'
INHIBIT_SYMMETRIC_SWAPPING = '&#8298'
ACTIVATE_SYMMETRIC_SWAPPING = '&#8299'
INHIBIT_ARABIC_FORM_SHAPING = '&#8300'
ACTIVATE_ARABIC_FORM_SHAPING = '&#8301'
NATIONAL_DIGIT_SHAPES = '&#8302'
NOMINAL_DIGIT_SHAPES = '&#8303'
