from . import HtmlButton
from . import HtmlContainer
from . import HtmlEvent
from . import HtmlFiles
from . import HtmlLinks
from . import HtmlImage
from . import HtmlInput
from . import HtmlList
from . import HtmlMenu
from . import HtmlOthers
from . import HtmlRadio
from . import HtmlSelect
from . import HtmlText
from . import HtmlTextComp
from . import HtmlTextEditor
from . import HtmlMedia
from . import Html
from . import HtmlPopup
from . import HtmlMessaging
from . import HtmlDates
from . import templates
from . import HtmlTrees
from . import HtmlTags
from . import HtmlStepper
from . import HtmlDrawer
from . import Header

from . import tables
from . import graph
from . import Defaults as Defaults_html
