"""
Related Pages:

		https://www.w3schools.com/charsets/ref_utf_geometric.asp
"""

BLACK_SQUARE = '&#9632'
WHITE_SQUARE = '&#9633'
WHITE_SQUARE_WITH_ROUNDED_CORNERS = '&#9634'
WHITE_SQUARE_CONTAINING_BLACK_SMALL_SQUARE = '&#9635'
SQUARE_WITH_HORIZONTAL_FILL = '&#9636'
SQUARE_WITH_VERTICAL_FILL = '&#9637'
SQUARE_WITH_ORTHOGONAL_CROSSHATCH_FILL = '&#9638'
SQUARE_WITH_UPPER_LEFT_TO_LOWER_RIGHT_FILL = '&#9639'
SQUARE_WITH_UPPER_RIGHT_TO_LOWER_LEFT_FILL = '&#9640'
SQUARE_WITH_DIAGONAL_CROSSHATCH_FILL = '&#9641'
BLACK_SMALL_SQUARE = '&#9642'
WHITE_SMALL_SQUARE = '&#9643'
BLACK_RECTANGLE = '&#9644'
WHITE_RECTANGLE = '&#9645'
BLACK_VERTICAL_RECTANGLE = '&#9646'
WHITE_VERTICAL_RECTANGLE = '&#9647'
BLACK_PARALLELOGRAM = '&#9648'
WHITE_PARALLELOGRAM = '&#9649'
BLACK_UP_POINTING_TRIANGLE = '&#9650'
WHITE_UP_POINTING_TRIANGLE = '&#9651'
BLACK_UP_POINTING_SMALL_TRIANGLE = '&#9652'
WHITE_UP_POINTING_SMALL_TRIANGLE = '&#9653'
BLACK_RIGHT_POINTING_TRIANGLE = '&#9654'
WHITE_RIGHT_POINTING_TRIANGLE = '&#9655'
BLACK_RIGHT_POINTING_SMALL_TRIANGLE = '&#9656'
WHITE_RIGHT_POINTING_SMALL_TRIANGLE = '&#9657'
BLACK_RIGHT_POINTING_POINTER = '&#9658'
WHITE_RIGHT_POINTING_POINTER = '&#9659'
BLACK_DOWN_POINTING_TRIANGLE = '&#9660'
WHITE_DOWN_POINTING_TRIANGLE = '&#9661'
BLACK_DOWN_POINTING_SMALL_TRIANGLE = '&#9662'
WHITE_DOWN_POINTING_SMALL_TRIANGLE = '&#9663'
BLACK_LEFT_POINTING_TRIANGLE = '&#9664'
WHITE_LEFT_POINTING_TRIANGLE = '&#9665'
BLACK_LEFT_POINTING_SMALL_TRIANGLE = '&#9666'
WHITE_LEFT_POINTING_SMALL_TRIANGLE = '&#9667'
BLACK_LEFT_POINTING_POINTER = '&#9668'
WHITE_LEFT_POINTING_POINTER = '&#9669'
BLACK_DIAMOND = '&#9670'
WHITE_DIAMOND = '&#9671'
WHITE_DIAMOND_CONTAINING_BLACK_SMALL_DIAMOND = '&#9672'
FISHEYE = '&#9673'
LOZENGE = '&#9674'
WHITE_CIRCLE = '&#9675'
DOTTED_CIRCLE = '&#9676'
CIRCLE_WITH_VERTICAL_FILL = '&#9677'
BULLSEYE = '&#9678'
BLACK_CIRCLE = '&#9679'
CIRCLE_WITH_LEFT_HALF_BLACK = '&#9680'
CIRCLE_WITH_RIGHT_HALF_BLACK = '&#9681'
CIRCLE_WITH_LOWER_HALF_BLACK = '&#9682'
CIRCLE_WITH_UPPER_HALF_BLACK = '&#9683'
CIRCLE_WITH_UPPER_RIGHT_QUADRANT_BLACK = '&#9684'
CIRCLE_WITH_ALL_BUT_UPPER_LEFT_QUADRANT_BLACK = '&#9685'
LEFT_HALF_BLACK_CIRCLE = '&#9686'
RIGHT_HALF_BLACK_CIRCLE = '&#9687'
INVERSE_BULLET = '&#9688'
INVERSE_WHITE_CIRCLE = '&#9689'
UPPER_HALF_INVERSE_WHITE_CIRCLE = '&#9690'
LOWER_HALF_INVERSE_WHITE_CIRCLE = '&#9691'
UPPER_LEFT_QUADRANT_CIRCULAR_ARC = '&#9692'
UPPER_RIGHT_QUADRANT_CIRCULAR_ARC = '&#9693'
LOWER_RIGHT_QUADRANT_CIRCULAR_ARC = '&#9694'
LOWER_LEFT_QUADRANT_CIRCULAR_ARC = '&#9695'
UPPER_HALF_CIRCLE = '&#9696'
LOWER_HALF_CIRCLE = '&#9697'
BLACK_LOWER_RIGHT_TRIANGLE = '&#9698'
BLACK_LOWER_LEFT_TRIANGLE = '&#9699'
BLACK_UPPER_LEFT_TRIANGLE = '&#9700'
BLACK_UPPER_RIGHT_TRIANGLE = '&#9701'
WHITE_BULLET = '&#9702'
SQUARE_WITH_LEFT_HALF_BLACK = '&#9703'
SQUARE_WITH_RIGHT_HALF_BLACK = '&#9704'
SQUARE_WITH_UPPER_LEFT_DIAGONAL_HALF_BLACK = '&#9705'
SQUARE_WITH_LOWER_RIGHT_DIAGONAL_HALF_BLACK = '&#9706'
WHITE_SQUARE_WITH_VERTICAL_BISECTING_LINE = '&#9707'
WHITE_UP_POINTING_TRIANGLE_WITH_DOT = '&#9708'
UP_POINTING_TRIANGLE_WITH_LEFT_HALF_BLACK = '&#9709'
UP_POINTING_TRIANGLE_WITH_RIGHT_HALF_BLACK = '&#9710'
LARGE_CIRCLE = '&#9711'
WHITE_SQUARE_WITH_UPPER_LEFT_QUADRANT = '&#9712'
WHITE_SQUARE_WITH_LOWER_LEFT_QUADRANT = '&#9713'
WHITE_SQUARE_WITH_LOWER_RIGHT_QUADRANT = '&#9714'
WHITE_SQUARE_WITH_UPPER_RIGHT_QUADRANT = '&#9715'
WHITE_CIRCLE_WITH_UPPER_LEFT_QUADRANT = '&#9716'
WHITE_CIRCLE_WITH_LOWER_LEFT_QUADRANT = '&#9717'
WHITE_CIRCLE_WITH_LOWER_RIGHT_QUADRANT = '&#9718'
WHITE_CIRCLE_WITH_UPPER_RIGHT_QUADRANT = '&#9719'
UPPER_LEFT_TRIANGLE = '&#9720'
UPPER_RIGHT_TRIANGLE = '&#9721'
LOWER_LEFT_TRIANGLE = '&#9722'
WHITE_MEDIUM_SQUARE = '&#9723'
BLACK_MEDIUM_SQUARE = '&#9724'
WHITE_MEDIUM_SMALL_SQUARE = '&#9725'
BLACK_MEDIUM_SMALL_SQUARE = '&#9726'
LOWER_RIGHT_TRIANGLE = '&#9727'